/***********************************************************************************************************************
 *
 * blueMarine Metadata - open source media workflow
 * Copyright (C) 2007-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluemarine.tidalwave.it
 * SCM: https://kenai.com/hg/bluemarine~metadata-src
 *
 **********************************************************************************************************************/
package it.tidalwave.metadata.persistence;

import java.util.List;
import org.junit.Before;
import org.junit.Test;
import static org.junit.Assert.*;

/*******************************************************************************
 * 
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 ******************************************************************************/
public class MetadataCompositeTest 
  {
    private MetadataComposite parent1;
    private MetadataComposite parent2;
    private MetadataComposite metadataComposite1;
    private MetadataComposite metadataComposite2;
    private MetadataComposite metadataComposite3;
    
    @Before
    public void setupFixture()
      {
        parent1 = new MetadataComposite(String.class);
        parent2 = new MetadataComposite(Integer.class);
        metadataComposite1 = new MetadataComposite(parent1, "property1");
        metadataComposite2 = new MetadataComposite(parent1, "property2");
        metadataComposite3 = new MetadataComposite(parent2, "property1");
      }

    @Before
    public void tearDownFixture()
      {
        parent1 = null;
        parent2 = null;
        metadataComposite1 = null;
        metadataComposite2 = null;
        metadataComposite3 = null;
      }
    
    @Test
    public void testGetItemClass()
      {
        assertEquals(String.class, parent1.getItemClass());
        assertEquals(Integer.class, parent2.getItemClass());
        assertEquals(String.class, metadataComposite1.getItemClass());
        assertEquals(String.class, metadataComposite2.getItemClass());
        assertEquals(Integer.class, metadataComposite3.getItemClass());
      }
    
    @Test
    public void testGetParent()
      {
        assertNull(parent1.getParent());
        assertNull(parent2.getParent());
        assertTrue(parent1 == metadataComposite1.getParent());
        assertTrue(parent1 == metadataComposite2.getParent());
        assertTrue(parent2 == metadataComposite3.getParent());
      }
    
    @Test
    public void testGetChildProperties()
      {
        final List<MetadataProperty> children = metadataComposite1.getChildProperties();
        assertEquals(2, children.size());
        assertEquals("child 1", children.get(0).getPropertyName());
        assertEquals("child 2", children.get(1).getPropertyName());
      }
    
    @Test
    public void testEquals()
      {
        final MetadataComposite parent1b = new MetadataComposite(String.class);
        final MetadataComposite parent2b = new MetadataComposite(Integer.class);
 
        final MetadataComposite metadataComposite1b = new MetadataComposite(parent1, "property1");
        final MetadataComposite metadataComposite2b = new MetadataComposite(parent1, "property2");
        final MetadataComposite metadataComposite3b = new MetadataComposite(parent2, "property1");
        
        assertTrue(parent1.equals(parent1b));
        assertTrue(parent1b.equals(parent1));
        assertTrue(parent2.equals(parent2b));
        assertTrue(parent2b.equals(parent2));
        assertFalse(parent1.equals(parent2));
        assertFalse(parent2.equals(parent1));
        
        assertFalse(parent1.equals(String.class));
        assertFalse(parent2.equals(Integer.class));
        assertFalse(metadataComposite1.equals(null));
        
        assertTrue(metadataComposite1.equals(metadataComposite1b));
        assertTrue(metadataComposite1b.equals(metadataComposite1));
        assertTrue(metadataComposite2.equals(metadataComposite2b));
        assertTrue(metadataComposite2b.equals(metadataComposite2));
        assertTrue(metadataComposite3.equals(metadataComposite3b));
        assertTrue(metadataComposite3b.equals(metadataComposite3));
        
        assertFalse(metadataComposite1.equals(metadataComposite2));
        assertFalse(metadataComposite2.equals(metadataComposite1));
        assertFalse(metadataComposite1.equals(metadataComposite3));
        assertFalse(metadataComposite3.equals(metadataComposite1));
        assertFalse(metadataComposite2.equals(metadataComposite3));
        assertFalse(metadataComposite3.equals(metadataComposite2));
      }

    @Test
    public void testHashCode()
      {
        final MetadataComposite parent1b = new MetadataComposite(String.class);
        final MetadataComposite parent2b = new MetadataComposite(Integer.class);
 
        final MetadataComposite metadataComposite1b = new MetadataComposite(parent1, "property1");
        final MetadataComposite metadataComposite2b = new MetadataComposite(parent1, "property2");
        final MetadataComposite metadataComposite3b = new MetadataComposite(parent2, "property1");

        assertEquals(parent1.hashCode(), parent1b.hashCode());
        assertEquals(parent2.hashCode(), parent2b.hashCode());
        assertEquals(metadataComposite1.hashCode(), metadataComposite1b.hashCode());
        assertEquals(metadataComposite2.hashCode(), metadataComposite2b.hashCode());
        assertEquals(metadataComposite3.hashCode(), metadataComposite3b.hashCode());
      }
    
    @Test
    public void testConstructorWithNullParent() 
      {
        try
          {
            new MetadataComposite(null, "property");
            fail("IllegalArgumentException expected");
          }
        catch (IllegalArgumentException e)
          {
            // ok
          }
      }   
    
    @Test
    public void testConstructorWithNullPropertyName() 
      {
        try
          {
            new MetadataComposite(parent1, null);
            fail("IllegalArgumentException expected");
          }
        catch (IllegalArgumentException e)
          {
            // ok
          }
      }   
    
    @Test
    public void testConstructorWithNullItemClass() 
      {
        try
          {
            new MetadataComposite(null);
            fail("IllegalArgumentException expected");
          }
        catch (IllegalArgumentException e)
          {
            // ok
          }
      }   
  }