/***********************************************************************************************************************
 *
 * blueMarine Metadata - open source media workflow
 * Copyright (C) 2007-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluemarine.tidalwave.it
 * SCM: https://kenai.com/hg/bluemarine~metadata-src
 *
 **********************************************************************************************************************/
package it.tidalwave.metadata.persistence;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.List;
import it.tidalwave.metadata.mock.MockItem1;
import it.tidalwave.metadata.persistence.PropertyAndValue.TemporarilyUnavailableException;
import it.tidalwave.metadata.persistence.mock.AssertingMetadataPersistenceMock;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import static org.junit.Assert.*;

/*******************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 ******************************************************************************/
public class PropertyAndValueTest 
  {
    private MetadataPropertySet set;
    private MetadataProperty<Object> property1;
    private MetadataProperty<Object> property2;
    private MetadataProperty<Object> property3;
    private PropertyAndValue<Object> pav1;
    private PropertyAndValue<Object> pav2;
    private PropertyAndValue<Object> pav3;
    private PropertyAndValue<Object> pav4;
    private PropertyAndValue<Object> pav5;
    
    @Before
    public void setupFixture()
      {
        set = new MetadataPropertySet(MockItem1.class);
        property1 = new MetadataProperty<Object>(set, "TAG1");
        property2 = new MetadataProperty<Object>(set, "TAG2");
        property3 = new MetadataProperty<Object>(set, "TAG3");
        pav1 = property1.findOrCreatePropertyAndValue("string");
        pav2 = property2.findOrCreatePropertyAndValue(2);
        pav3 = property3.findOrCreatePropertyAndValue(null);
        pav4 = property3.findOrCreatePropertyAndValue("value");
        pav5 = property3.findOrCreatePropertyAndValue("value2");
      }
    
    @After
    public void tearDownFixture()
      {
        set = null;
        property1 = null;
        property2 = null;
        property3 = null;
        pav1 = null;
        pav2 = null;
        pav3 = null;
        pav4 = null;
        pav5 = null;
      }
    
    @Test
    public void testGetObject()
      {
        assertEquals("string", pav1.getValue());
        assertTrue(property1 == pav1.getProperty());
        assertEquals(2, pav2.getValue());
        assertTrue(property2 == pav2.getProperty());
        assertEquals(null, pav3.getValue());
        assertTrue(property3 == pav3.getProperty());
      }
    
    @Test
    public void testGetDataObjectCount()
      throws InterruptedException
      {
        final List<PropertyChangeEvent> events = new ArrayList<PropertyChangeEvent>();
        
        final PropertyChangeListener propertyChangeListener = new PropertyChangeListener() 
          {
            public void propertyChange (final PropertyChangeEvent event) 
              {
                events.add(event);
              }
          };
        
        final AssertingMetadataPersistenceMock persistence = (AssertingMetadataPersistenceMock)MetadataPersistence.Locator.findPersistence();
        pav1.addPropertyChangeListener(propertyChangeListener);
        
        try
          {
            pav1.getDataObjectCount();
            fail("TemporaryUnavailableException expected");
          }
        catch (TemporarilyUnavailableException e)
          {
            // ok    
          }
        
//        assertEquals("...", persistence.latestFindBindCountCriterion.toString());
        assertTrue(events.isEmpty());
        persistence.setDataObjectCountResult(17);
        Thread.sleep(1000);
        assertEquals(1, events.size());
        final PropertyChangeEvent event = events.get(0);
        assertEquals(pav1, event.getSource());
        assertEquals("dataObjectCount", event.getPropertyName());
        assertEquals(17, event.getNewValue());
        assertEquals(null, event.getOldValue());
        
        try
          {
            assertEquals(17, pav1.getDataObjectCount());
          }
        catch (TemporarilyUnavailableException e)
          {
            fail("TemporaryUnavailableException unexpected now");
          }
      }
    
    @Test
    public void testToString()
      {
        assertEquals("PropertyAndValue[TAG1=string]", pav1.toString());
        assertEquals("PropertyAndValue[TAG2=2]", pav2.toString());
        assertEquals("PropertyAndValue[TAG3=null]", pav3.toString());
      }
        
    @Test
    public void testEquals()
      {
        final PropertyAndValue pav1b = property1.findOrCreatePropertyAndValue("string");
        final PropertyAndValue pav2b = property2.findOrCreatePropertyAndValue(2);
        final PropertyAndValue pav3b = property3.findOrCreatePropertyAndValue(null);
        
        assertTrue(pav1.equals(pav1b));
        assertTrue(pav1b.equals(pav1));
        assertTrue(pav2.equals(pav2b));
        assertTrue(pav2b.equals(pav2));
        assertTrue(pav3.equals(pav3b));
        assertTrue(pav3b.equals(pav3));
        
        assertFalse(pav1.equals(pav2b));
        assertFalse(pav1b.equals(pav2));
        assertFalse(pav1.equals(pav3b));
        assertFalse(pav1b.equals(pav3));
        assertFalse(pav2.equals(pav3b));
        assertFalse(pav2b.equals(pav3));
        
        assertFalse(pav1.equals(null));
        
        assertFalse(pav3.equals(pav4));
        assertFalse(pav4.equals(pav3));
        assertFalse(pav3.equals(pav5));
        assertFalse(pav5.equals(pav3));
        assertFalse(pav4.equals(pav5));
        assertFalse(pav5.equals(pav4));
        
        assertFalse(pav1.equals("string"));
      }
    
    @Test
    public void testHashcode()
      {
        final PropertyAndValue pav1b = property1.findOrCreatePropertyAndValue("string");
        final PropertyAndValue pav2b = property2.findOrCreatePropertyAndValue(2);
        final PropertyAndValue pav3b = property3.findOrCreatePropertyAndValue(null);
        
        assertEquals(pav1.hashCode(), pav1b.hashCode());    
        assertEquals(pav2.hashCode(), pav2b.hashCode());    
        assertEquals(pav3.hashCode(), pav3b.hashCode());    
      }
  }