/***********************************************************************************************************************
 *
 * blueMarine Metadata - open source media workflow
 * Copyright (C) 2007-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluemarine.tidalwave.it
 * SCM: https://kenai.com/hg/bluemarine~metadata-src
 *
 **********************************************************************************************************************/
package it.tidalwave.metadata.persistence.spi;

import java.util.List;
import java.util.Arrays;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import static org.junit.Assert.*;

class CodeBasedBeanAccessorFixture extends CodeBasedBeanAccessor<MapBasedBeanMock>    
  {
    public final static List<String> TRANSIENT_PROPERTIES = Arrays.asList("tprop1", "tprop2");
    
    public CodeBasedBeanAccessorFixture()
      {
        super(MapBasedBeanMock.class, "group", TRANSIENT_PROPERTIES);
      }
    
    public List<String> getPersistentPropertyNames (final MapBasedBeanMock bean)   
      {
        return Arrays.asList("property1", "property2", "unmappedProperty");
      }

    @Override
    protected Object getProperty (final MapBasedBeanMock bean, final int code)
      {
        return bean.getObject(code);
      }

    @Override
    protected void setProperty (final MapBasedBeanMock bean, final int code, final Object value) 
      {
        bean.setObject(code, value);
      }
    
    protected void registerCodes()
      {
        registerCode("property1", 1);
        registerCode("property2", 2);
        registerCode("tprop1", 3);
        registerCode("tprop2", 4);
      }
  }

/*******************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 ******************************************************************************/
public class CodeBasedBeanAccessorTest 
  {
    private CodeBasedBeanAccessorFixture beanAccessor;
    
    @Before
    public void setupFixture() 
      {
        beanAccessor = new CodeBasedBeanAccessorFixture();
      }

    @After
    public void tearDownFixture() 
      {
        beanAccessor = null;
      }

    @Test
    public void testIsPersistent() 
      {
        assertTrue(beanAccessor.isPersistent("property1"));
        assertTrue(beanAccessor.isPersistent("property2"));
        assertTrue(beanAccessor.isPersistent("unmappedProperty"));
        assertFalse(beanAccessor.isPersistent("tprop1"));
        assertFalse(beanAccessor.isPersistent("tprop2"));
      }

    @Test
    public void testGetPropertySetName() 
      {
        assertEquals("group", beanAccessor.getPropertySetName());
      }

    @Test
    public void testConvertValueToPersistence() 
      throws UnknownPropertyException
      {
        assertEquals(1, beanAccessor.convertValueToPersistence("property1", 1));
        assertEquals("test", beanAccessor.convertValueToPersistence("property2", "test"));
      }

    @Test
    public void testConvertValueFromPersistence() 
      throws UnknownPropertyException
      {
        assertEquals(1, beanAccessor.convertValueFromPersistence("property1", 1));
        assertEquals("test", beanAccessor.convertValueFromPersistence("property2", "test"));
      }

    @Test
    public void testConvertPropertyNameToPersistence() 
      {
        assertEquals("#1", beanAccessor.convertPropertyNameToPersistence("property1"));
        assertEquals("#2", beanAccessor.convertPropertyNameToPersistence("property2"));
        assertEquals("unmappedProperty", beanAccessor.convertPropertyNameToPersistence("unmappedProperty"));
      }

    @Test
    public void testConvertPropertyNameFromPersistence()
      {
        assertEquals("property1", beanAccessor.convertPropertyNameFromPersistence("#1"));
        assertEquals("property2", beanAccessor.convertPropertyNameFromPersistence("#2"));
        assertEquals("unmappedProperty", beanAccessor.convertPropertyNameFromPersistence("unmappedProperty"));
      }
    
    @Test
    public void testGetProperty()
      throws UnknownPropertyException
      {
        final MapBasedBeanMock bean = new MapBasedBeanMock();
        bean.setObject(1, "value 1");
        bean.setObject(2, "value 2");
        bean.setUnmappedProperty("value 3");
        assertEquals("value 1", beanAccessor.getProperty("#1", bean));
        assertEquals("value 2", beanAccessor.getProperty("#2", bean));
        assertEquals("value 3", beanAccessor.getProperty("unmappedProperty", bean));
      }
    
    @Test
    public void testSetProperty()
      throws UnknownPropertyException
      {
        final MapBasedBeanMock bean = new MapBasedBeanMock();
        beanAccessor.setProperty(bean, "#1", "value 1");
        beanAccessor.setProperty(bean, "#2", "value 2");
        beanAccessor.setProperty(bean, "unmappedProperty", "value 3");
        assertEquals("value 1", bean.getObject(1));
        assertEquals("value 2", bean.getObject(2)); 
        assertEquals("value 3", bean.getUnmappedProperty());
      }
    
    @Test
    public void testGetDefaultPropertyType()
      throws UnknownPropertyException
      {
        // has a code, wrapped by a JavaBean property
        assertEquals(Integer.class, beanAccessor.getDefaultPropertyType("#1")); 
        // doesn't have a code, wrapped by a JavaBean property
        assertEquals(String.class, beanAccessor.getDefaultPropertyType("unmappedProperty"));

        try
          {
            // this isn't wrapped by a property, so we can't guess the type
            beanAccessor.getDefaultPropertyType("#2");
            fail("UnknownPropertyException expected");
          }
        catch (UnknownPropertyException e)
          {
            // ok, expected
          }

        try
          {
            // completely unknown code
            beanAccessor.getDefaultPropertyType("#222"); 
            fail("UnknownPropertyException expected");
          }
        catch (UnknownPropertyException e)
          {
            // ok, expected
          }
      }
  }